/*
  Doc_Arduino - german Arduino Forum
  Programmed and tested with: Arduino IDE 1.8.13
  16.03.2021
  License: GNU GPLv3
*/

#include <NanoEveryPin.h>

OutputPin   <13> onboardLed;
AnalogInPin <21> analogIn;  // PD5

void setup()
{
  Serial.begin(250000);
  Serial.println(F("\nStart #### #### #### ####"));
  
  onboardLed.init();
  analogIn.init();

  initRTC();
  initADC0();
  initEventSystem();
}

void loop()
{
  readingADC0();
}

void readingADC0 (void)
{
  if (ADC0.INTFLAGS & ADC_RESRDY_bm)
  {
    const unsigned int reading = ADC0.RES;
    Serial.print("ADC0: "); Serial.print(reading); Serial.print(" Digits");
    // wenn man davon ausgeht das Ub 5.0V sind ...
    const unsigned long voltage = 5000UL * reading / 1024;
    Serial.print(" = "); Serial.print(voltage); Serial.println("mV");
    ADC0.INTFLAGS = ADC_RESRDY_bm;      // clear result ready interrupt flag
    onboardLed.toggle();
  }
}

void initADC0(void)
{
  ADC0.MUXPOS = ADC_MUXPOS_AIN5_gc;     // select ADC0 input channel -->> Pin 21 / PD5
  ADC0.EVCTRL = ADC_STARTEI_bm;         // enable event triggered conversion
}

void initRTC (void)
{
  while (RTC.STATUS > 0) { ; }          // wait for all register to be synchronized
  RTC.CLKSEL = RTC_CLKSEL_INT32K_gc;    // period duration 30,518µs
  RTC.PER = 64;                         // set period = 1000ms / 15,625ms
  
  // RTC.CTRLA Bug, see Errata
  byte temp {0};
  temp |= RTC_PRESCALER_DIV512_gc;      // prescaler 512 -->> 30,518µs * 512 = 15,625ms
  temp |= RTC_RTCEN_bm;                 // RTC enable  
  RTC.CTRLA = temp;
}

void initEventSystem(void)
{
  EVSYS.CHANNEL7 = EVSYS_GENERATOR_RTC_OVF_gc;    // connect 'RTC OVF' to channel 7
  EVSYS.USERADC0 = EVSYS_CHANNEL_CHANNEL7_gc;     // connect generator 'RTC OVF' with user 'ADC0' via channel 7
}
